package immibis.ccperiphs;

import net.minecraft.src.*;
import net.minecraft.src.forge.IGuiHandler;
import net.minecraft.src.forge.MinecraftForge;
import immibis.ccperiphs.rfid.*;
import immibis.core.CompatibleBaseMod;
import immibis.core.Config;
import immibis.core.CoreProxy;
import immibis.core.IBlockIDCallback;
import immibis.core.ModInfoReader;
import immibis.core.NonSharedProxy;

public class mod_ImmibisPeripherals extends CompatibleBaseMod implements IGuiHandler {
	
	public static BlockPeriphs block;
	public static ItemCardBase itemRFID, itemMagStripe;
	public static mod_ImmibisPeripherals instance;
	
	public static final int GUI_RFID_WRITER = 0;
	
	@Override
	public String getPriorities() {
		return "after:mod_ImmibisCore";
	}
	
	public mod_ImmibisPeripherals() {
		instance = this;
	}

	@Override
	public String getVersion() {
		return ModInfoReader.getModInfoField("/immibis/ccperiphs/mod_ImmibisPeripherals.info.txt", "version");
	}

	@SuppressWarnings("unchecked")
	@Override
	public void load() {
		CoreProxy.RegisterBlockID("peripherals.block", new IBlockIDCallback() {
			@Override
			public void registerBlock(int id) {
				block = new BlockPeriphs(id);
				ModLoader.registerBlock(block, ItemPeriphs.class);
			}
		});
		
		BlockPeriphs.model = NonSharedProxy.getUniqueBlockModelID(this, true);
		
		itemRFID = new ItemCardBase(Config.getItemID("peripherals.rfid", 7434) - 256, 48);
		itemMagStripe = new ItemCardBase(Config.getItemID("peripherals.magstripe", 7435) - 256, 64);
		
		ModLoader.addName(itemRFID, "RFID card");
		ModLoader.addName(itemMagStripe, "Mag-stripe card");
		
		CraftingManager.getInstance().getRecipeList().add(new DyeCardRecipe(itemRFID.shiftedIndex));
		CraftingManager.getInstance().getRecipeList().add(new DyeCardRecipe(itemMagStripe.shiftedIndex));
		
		/* $if client$ */
		ModLoader.registerTileEntity(TileRFIDWriter.class, "immibis.cc-rfid.writer", new RenderTileRFIDWriter());
		/* $else$ */
		ModLoader.registerTileEntity(TileRFIDWriter.class, "immibis.cc-rfid.writer");
		/* $endif$ */
		
		ModLoader.registerTileEntity(TileRFIDReader.class, "immibis.cc-rfid.reader");
		ModLoader.registerTileEntity(TileMagStripe.class, "immibis.cc-rfid.msreader");
		
		MinecraftForge.setGuiHandler(this, this);
	}
	
	@Override
	public void modsLoaded() {
		ModLoader.addRecipe(new ItemStack(itemRFID, 1, 0),
			"RRR",
			"PPP",
			"RRR",
			'P', Item.paper,
			'R', Item.redstone
		);
		ModLoader.addRecipe(new ItemStack(itemMagStripe, 1, 0),
			"PPP",
			"IRI",
			'P', Item.paper,
			'R', Item.redstone,
			'I', Item.ingotIron
		);
		ModLoader.addRecipe(new ItemStack(block, 1, EnumPeriphs.RFID_READER.ordinal()),
			" T ",
			"SIS",
			"SRS",
			'S', Block.stone,
			'T', Block.torchRedstoneActive,
			'R', Item.redstone,
			'I', Item.ingotIron
		);
		ModLoader.addRecipe(new ItemStack(block, 1, EnumPeriphs.RFID_WRITER.ordinal()),
			"L-L",
			"# #",
			"SRS",
			'S', Block.stone,
			'R', Item.redstoneRepeater,
			'#', Block.blockGold,
			'-', Block.thinGlass,
			'L', Block.redstoneLampIdle
		);
		ModLoader.addRecipe(new ItemStack(block, 1, EnumPeriphs.MAG_STRIPE.ordinal()),
			"STS",
			"SRS",
			"SSS",
			'S', Block.stone,
			'R', Item.redstone,
			'T', Block.torchRedstoneActive
		);
	}
	
	/* $if client$ */
	@Override
	public boolean renderWorldBlock(RenderBlocks render, IBlockAccess w, int x, int y, int z, Block block, int model) {
		TilePeriphs t = (TilePeriphs)w.getBlockTileEntity(x, y, z);
		if(t == null)
			return false;
		RenderUtils.renderBlockStatic(render, t.getBlockMetadata(), t, x, y, z);
		return true;
	}
	
	@Override
	public void renderInvBlock(RenderBlocks render, Block block, int meta, int model) {
		Tessellator.instance.startDrawingQuads();
		RenderUtils.renderBlockStatic(render, meta, null, -0.5, -0.5, -0.5);
		Tessellator.instance.draw();
		RenderUtils.renderInvBlock(render, meta);
	}
	/* $endif$ */

	@Override
	public Object getGuiElement(int ID, EntityPlayer player, World world, int x, int y, int z) {
		TileEntity tile = world.getBlockTileEntity(x, y, z);
		switch(ID) {
		/* $if client$ */
		case GUI_RFID_WRITER: return new GuiRFIDWriter(new ContainerRFIDWriter(player, (TileRFIDWriter)tile));
		/* $else$
		case GUI_RFID_WRITER: return new ContainerRFIDWriter(player, (TileRFIDWriter)tile);
		$endif$ */
		}
		return null;
	}
	
}
